"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DEFAULT_OPTIONS = void 0;
exports.DEFAULT_OPTIONS = Object.freeze({
    allClearResetTime: 5000,
    connectTimeout: 5000,
    debug: false,
    minReconnectDelay: 1000,
    maxReconnectDelay: 30000,
    maxReconnectAttempts: Number.POSITIVE_INFINITY,
    reconnectBackoffFactor: 1.5,
});
const CONNECTING = 0;
const OPEN = 1;
const CLOSING = 2;
const CLOSED = 3;
class SturdyWebSocket {
    constructor(url, protocolsOrOptions, options = {}) {
        this.onclose = null;
        this.onerror = null;
        this.onmessage = null;
        this.onopen = null;
        /**
         * Called when the backing WebSocket is closed but `SturdyWebSocket` will try to reconnect.
         * Recieves the `CloseEvent` of the backing WebSocket.
         * If this was triggered by a call to `reconnect()`, then the event will be `undefined`.
         */
        this.ondown = null;
        /**
         * Called when the backing WebSocket is reopened after it closed.
         */
        this.onreopen = null;
        this.CONNECTING = CONNECTING;
        this.OPEN = OPEN;
        this.CLOSING = CLOSING;
        this.CLOSED = CLOSED;
        this.hasBeenOpened = false;
        this.isClosed = false;
        this.messageBuffer = [];
        this.nextRetryTime = 0;
        this.reconnectCount = 0;
        this.lastKnownExtensions = "";
        this.lastKnownProtocol = "";
        this.listeners = {};
        this.url = url;
        if (protocolsOrOptions == null ||
            typeof protocolsOrOptions === "string" ||
            Array.isArray(protocolsOrOptions)) {
            this.protocols = protocolsOrOptions;
        }
        else {
            options = protocolsOrOptions;
        }
        this.options = Object.assign({}, exports.DEFAULT_OPTIONS, options);
        this.openNewWebSocket();
    }
    get binaryType() {
        return this.binaryTypeInternal || "blob";
    }
    set binaryType(binaryType) {
        this.binaryTypeInternal = binaryType;
        if (this.ws) {
            this.ws.binaryType = binaryType;
        }
    }
    get bufferedAmount() {
        let sum = this.ws ? this.ws.bufferedAmount : 0;
        let hasUnknownAmount = false;
        this.messageBuffer.forEach((data) => {
            const byteLength = getDataByteLength(data);
            if (byteLength != null) {
                sum += byteLength;
            }
            else {
                hasUnknownAmount = true;
            }
        });
        if (hasUnknownAmount) {
            this.debugLog("Some buffered data had unknown length. bufferedAmount()" +
                " return value may be below the correct amount.");
        }
        return sum;
    }
    get extensions() {
        return this.ws ? this.ws.extensions : this.lastKnownExtensions;
    }
    get protocol() {
        return this.ws ? this.ws.protocol : this.lastKnownProtocol;
    }
    get readyState() {
        return this.isClosed ? CLOSED : OPEN;
    }
    close(code, reason) {
        this.disposeSocket(code, reason);
        this.shutdown();
        this.debugLog("WebSocket permanently closed by client.");
    }
    send(data) {
        if (this.ws && this.ws.readyState === this.OPEN) {
            this.ws.send(data);
        }
        else {
            this.messageBuffer.push(data);
        }
    }
    /**
     * Closes the backing websocket and opens a new one.
     * This will immediately call the `down` handler with no event,
     * followed by the `reopen` handler once the connection is reestablished.
     */
    reconnect() {
        if (this.isClosed) {
            throw new Error("Cannot call reconnect() on socket which is permanently closed.");
        }
        this.disposeSocket(1000, "Client requested reconnect.");
        this.handleClose(undefined);
    }
    addEventListener(type, listener) {
        if (!this.listeners[type])
            this.listeners[type] = [];
        this.listeners[type].push(listener);
    }
    dispatchEvent(event) {
        return this.dispatchEventOfType(event.type, event);
    }
    removeEventListener(type, listener) {
        if (this.listeners[type]) {
            this.listeners[type] = this.listeners[type].filter((l) => l !== listener);
        }
    }
    openNewWebSocket() {
        if (this.isClosed)
            return;
        const { connectTimeout } = this.options;
        this.debugLog(`Opening new WebSocket to ${this.url}.`);
        const ws = new WebSocket(this.url, this.protocols);
        ws.onclose = (event) => this.handleClose(event);
        ws.onerror = (event) => this.handleError(event);
        ws.onmessage = (event) => this.handleMessage(event);
        ws.onopen = (event) => this.handleOpen(event);
        this.connectTimeoutId = setTimeout(() => {
            // If this is running, we still haven't opened the websocket.
            // Kill it so we can try again.
            this.clearConnectTimeout();
            this.disposeSocket();
            this.handleClose(undefined);
        }, connectTimeout);
        this.ws = ws;
    }
    handleOpen(event) {
        if (!this.ws || this.isClosed)
            return;
        const { allClearResetTime } = this.options;
        this.debugLog("WebSocket opened.");
        if (this.binaryTypeInternal != null) {
            this.ws.binaryType = this.binaryTypeInternal;
        }
        else {
            this.binaryTypeInternal = this.ws.binaryType;
        }
        this.clearConnectTimeout();
        if (this.hasBeenOpened) {
            this.dispatchEventOfType("reopen", event);
            // Note that buffered messages from while the connection was down is not sent.
            // We need to re-initialize the Language Server state.
        }
        else {
            this.dispatchEventOfType("open", event);
            this.hasBeenOpened = true;
            this.messageBuffer.forEach((message) => this.send(message));
        }
        this.messageBuffer = [];
        this.allClearTimeoutId = setTimeout(() => {
            this.clearAllClearTimeout();
            this.nextRetryTime = 0;
            this.reconnectCount = 0;
            const openTime = (allClearResetTime / 1000) | 0;
            this.debugLog(`WebSocket remained open for ${openTime}s. Resetting retry time and count`);
        }, allClearResetTime);
    }
    handleMessage(event) {
        if (this.isClosed)
            return;
        this.dispatchEventOfType("message", event);
    }
    handleClose(event) {
        if (this.isClosed)
            return;
        const { maxReconnectAttempts } = this.options;
        this.clearConnectTimeout();
        this.clearAllClearTimeout();
        if (this.ws) {
            this.lastKnownExtensions = this.ws.extensions;
            this.lastKnownProtocol = this.ws.protocol;
            this.ws = undefined;
        }
        this.dispatchEventOfType("down", event);
        if (this.reconnectCount >= maxReconnectAttempts) {
            const reason = `Failed to reconnect after ${maxReconnectAttempts} attempts. Closing permanently.`;
            this.stopReconnecting(event, reason);
        }
        else {
            this.reestablishConnection();
        }
    }
    handleError(event) {
        this.dispatchEventOfType("error", event);
        this.debugLog("WebSocket encountered an error.");
    }
    reestablishConnection() {
        const { minReconnectDelay, maxReconnectDelay, reconnectBackoffFactor, } = this.options;
        this.reconnectCount++;
        const retryTime = this.nextRetryTime;
        this.nextRetryTime = Math.max(minReconnectDelay, Math.min(this.nextRetryTime * reconnectBackoffFactor, maxReconnectDelay));
        setTimeout(() => this.openNewWebSocket(), retryTime);
        const retryTimeSeconds = (retryTime / 1000) | 0;
        this.debugLog(`WebSocket was closed. Re-opening in ${retryTimeSeconds}s.`);
    }
    stopReconnecting(event, debugReason) {
        this.debugLog(debugReason);
        this.shutdown();
        if (event) {
            this.dispatchEventOfType("close", event);
        }
    }
    shutdown() {
        this.isClosed = true;
        this.clearAllTimeouts();
        this.messageBuffer = [];
    }
    disposeSocket(closeCode, reason) {
        if (!this.ws)
            return;
        this.ws.onerror = noop;
        this.ws.onclose = noop;
        this.ws.onmessage = noop;
        this.ws.onopen = noop;
        this.ws.close(closeCode, reason);
        this.ws = undefined;
    }
    clearAllTimeouts() {
        this.clearConnectTimeout();
        this.clearAllClearTimeout();
    }
    clearConnectTimeout() {
        if (this.connectTimeoutId != null) {
            clearTimeout(this.connectTimeoutId);
            this.connectTimeoutId = undefined;
        }
    }
    clearAllClearTimeout() {
        if (this.allClearTimeoutId != null) {
            clearTimeout(this.allClearTimeoutId);
            this.allClearTimeoutId = undefined;
        }
    }
    dispatchEventOfType(type, event) {
        switch (type) {
            case "close":
                if (this.onclose)
                    this.onclose(event);
                break;
            case "error":
                if (this.onerror)
                    this.onerror(event);
                break;
            case "message":
                if (this.onmessage)
                    this.onmessage(event);
                break;
            case "open":
                if (this.onopen)
                    this.onopen(event);
                break;
            case "down":
                if (this.ondown)
                    this.ondown(event);
                break;
            case "reopen":
                if (this.onreopen)
                    this.onreopen(event);
                break;
        }
        if (type in this.listeners) {
            this.listeners[type]
                .slice()
                .forEach((listener) => this.callListener(listener, event));
        }
        return !event || !event.defaultPrevented;
    }
    callListener(listener, event) {
        if (typeof listener === "function") {
            listener.call(this, event);
        }
        else {
            listener.handleEvent.call(this, event);
        }
    }
    debugLog(message) {
        if (this.options.debug)
            console.debug(message);
    }
}
exports.default = SturdyWebSocket;
function getDataByteLength(data) {
    if (typeof data === "string") {
        // UTF-16 strings use two bytes per character.
        return 2 * data.length;
    }
    else if (data instanceof ArrayBuffer) {
        return data.byteLength;
    }
    else if (data instanceof Blob) {
        return data.size;
    }
    else {
        return undefined;
    }
}
const noop = () => { };
// MIT License
//
// Copyright (c) 2017 David Philipson
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
//# sourceMappingURL=ws.js.map