/// <reference types="codemirror/addon/hint/show-hint" />
import type { Editor } from "codemirror";
import { FileEvent } from "vscode-languageserver-protocol";
/**
 * Describes text document's language association.
 */
export interface LanguageAssociation {
    /**
     * Language ID associated with the file.
     */
    languageId: string;
    /**
     * IDs of language servers to connect to.
     * Accepts multiple IDs for future extension, but currently only the first one is used.
     */
    languageServerIds: string[];
}
/** Configs to use for each server. */
export interface LanguageServerConfigs {
    [serverId: string]: LanguageServerConfig | undefined;
}
/**
 * Server's `initializationOptions` and client `settings`.
 */
export interface LanguageServerConfig {
    /** Custom initialization options for the server. */
    initOptions?: any;
    /** Client settings for the server. */
    settings?: any;
}
/**
 * Options for Workspace.
 */
export interface WorkspaceOptions {
    /**
     * The URI of the project root.
     */
    rootUri: string;
    /**
     * Optional configs for Language Servers.
     */
    configs?: LanguageServerConfigs;
    /**
     * Provide Language Server connection string.
     *
     * The returned string can be either:
     *
     * - URI of a WebSocket proxy of a Language Server (`wss?://`)
     * - Path to a script to start Language Server in Web Worker
     *
     * If the returned string does not start with `wss?://`, it's assumed to be a Worker script.
     */
    getConnectionString: (this: void, langserverId: string) => Promise<string>;
    /**
     * Provide language association (language id, language server id) for a file with the uri.
     */
    getLanguageAssociation: (this: void, uri: string) => LanguageAssociation | null;
    /**
     * Function to render Markdown to HTML string.
     * If not provided and the server's response contains Markdown, it'll be displayed as is.
     */
    renderMarkdown?: (this: void, markdown: string) => string;
    /**
     * Optional callback called after a WebSocket connection goes down.
     */
    onDisconnected?: (serverId: string) => void;
    /**
     * Optional callback called after a WebSocket connection reopens and initialization completes.
     */
    onReconnected?: (serverId: string) => Promise<void>;
    /**
     * Optional callback called when a connecting to a Language Server failed.
     * WebSocket connection is retried up to 10 consecutive failures with exponential backoff.
     */
    onConnectionFailed?: (serverId: string) => void;
}
/**
 * Workspace provides code intelligence to CodeMirror editors by managing
 * communications with Language Servers and adding event handlers.
 */
export declare class Workspace {
    private editors;
    private connectionPromises;
    private connections;
    private documentVersions;
    private subscriptionDisposers;
    private getConnectionString;
    private getLanguageAssociation;
    private renderMarkdown;
    private canHandleMarkdown;
    private rootUri;
    private configs;
    private completionHandlers;
    private popupsEnabled;
    private onDisconnected;
    private onReconnected;
    private onConnectionFailed;
    private connectionToUris;
    /**
     * Create new workspace.
     * @param options
     */
    constructor(options: WorkspaceOptions);
    enablePopups(enabled?: boolean): void;
    /**
     * Dispose the workspace.
     * Close connections and remove references to editors.
     */
    dispose(): void;
    /**
     * Open text document in the workspace to notify the Language Server and
     * enable code intelligence.
     * @param path - The file path relative to the project root.
     * @param editor - CodeMirror Editor instance.
     */
    openTextDocument(path: string, editor: Editor): Promise<void>;
    private addEventHandlers;
    hideAllPopups(cm?: Editor): void;
    hideTooltips(cm?: Editor): void;
    /**
     * Close text document in the workspace to notify the Language Server and
     * remove everything added by the workspace.
     * @param path - The file path relative to the project root.
     */
    closeTextDocument(path: string): Promise<void>;
    private detachEditor;
    /**
     * Notify the Language Server that the text document was saved.
     *
     * If connected to a WebSocket proxy with synchronization enabled,
     * the contents of the file will be written to disk.
     * @param path - The file path relative to the project root.
     */
    saveTextDocument(path: string): Promise<void>;
    /**
     * Send `workspace/didChangeWatchedFiles` notification to all active Language Servers.
     *
     * Note that `FileEvent` must have a valid `uri` starting with `rootUri`.
     * @param changes
     */
    notifyFilesChanged(changes: FileEvent[]): Promise<void>;
    private getConnection;
    private removeEventHandlers;
    private initConnection;
    private getDocumentUri;
    private openConnection;
}
