export interface Options {
    /**
     * If a newly opened WebSocket closes immediately, it is considered to be a
     * failed connection for the purposes of increasing time between attempts and
     * counting towards `maxReconnectAttempts`.
     * This option controls how long a connection must remain open to be considered
     * "successful" and reset these values.
     * Default: 5000
     */
    allClearResetTime?: number;
    /**
     * When attempting to open a new connection, how long to wait before giving up and
     * making a new connection. Note that it is possible for an attempt to open a WebSocket
     * to stall forever, which is why this option is needed.
     * Default: 5000
     */
    connectTimeout?: number;
    /**
     * If true, print various debug information to console.log, such as notifying
     * about reconnect attempts.
     * Default: false
     */
    debug?: boolean;
    /**
     * The minimum positive time between failed reconnect attempts.
     * Note that the first reconnect attempt happens immediately on the first failure,
     * so this is actually the delay between the first and second reconnect attempts.
     * Default: 1000
     */
    minReconnectDelay?: number;
    /**
     * The maximum time between failed reconnect attempts.
     * Additional attempts will repeatedly use this as their delay.
     * Default: 30000
     */
    maxReconnectDelay?: number;
    /**
     * If reconnects fail this many times in a row, then the SturdyWebSocket closes permanently,
     * providing the CloseEvent from the last failed reconnect attempt.
     * Default: Infinity
     */
    maxReconnectAttempts?: number;
    /**
     * The factor by which the time between reconnect attempts increases after each failure.
     * Default: 1.5
     */
    reconnectBackoffFactor?: number;
}
interface SturdyWebSocketEventMap extends WebSocketEventMap {
    down: CloseEvent;
    reopen: Event;
}
export declare const DEFAULT_OPTIONS: Required<Options>;
export default class SturdyWebSocket implements WebSocket {
    onclose: ((event: CloseEvent) => void) | null;
    onerror: ((event: Event) => void) | null;
    onmessage: ((event: MessageEvent) => void) | null;
    onopen: ((event: Event) => void) | null;
    /**
     * Called when the backing WebSocket is closed but `SturdyWebSocket` will try to reconnect.
     * Recieves the `CloseEvent` of the backing WebSocket.
     * If this was triggered by a call to `reconnect()`, then the event will be `undefined`.
     */
    ondown: ((event: CloseEvent | undefined) => void) | null;
    /**
     * Called when the backing WebSocket is reopened after it closed.
     */
    onreopen: ((event: Event) => void) | null;
    readonly CONNECTING = 0;
    readonly OPEN = 1;
    readonly CLOSING = 2;
    readonly CLOSED = 3;
    private readonly protocols?;
    private readonly options;
    readonly url: string;
    private ws?;
    private hasBeenOpened;
    private isClosed;
    private messageBuffer;
    private nextRetryTime;
    private reconnectCount;
    private allClearTimeoutId?;
    private connectTimeoutId?;
    private binaryTypeInternal?;
    private lastKnownExtensions;
    private lastKnownProtocol;
    private readonly listeners;
    constructor(url: string, options?: Options);
    constructor(url: string, protocols: string | string[] | undefined, options?: Options);
    get binaryType(): BinaryType;
    set binaryType(binaryType: BinaryType);
    get bufferedAmount(): number;
    get extensions(): string;
    get protocol(): string;
    get readyState(): number;
    close(code?: number, reason?: string): void;
    send(data: any): void;
    /**
     * Closes the backing websocket and opens a new one.
     * This will immediately call the `down` handler with no event,
     * followed by the `reopen` handler once the connection is reestablished.
     */
    reconnect(): void;
    addEventListener<K extends keyof SturdyWebSocketEventMap>(type: K, listener: (this: WebSocket, event: SturdyWebSocketEventMap[K]) => void): void;
    addEventListener(type: string, listener: EventListenerOrEventListenerObject): void;
    dispatchEvent(event: Event): boolean;
    removeEventListener<K extends keyof SturdyWebSocketEventMap>(type: K, listener: (this: WebSocket, event: SturdyWebSocketEventMap[K]) => void): void;
    removeEventListener(type: string, listener: EventListenerOrEventListenerObject): void;
    private openNewWebSocket;
    private handleOpen;
    private handleMessage;
    private handleClose;
    private handleError;
    private reestablishConnection;
    private stopReconnecting;
    private shutdown;
    private disposeSocket;
    private clearAllTimeouts;
    private clearConnectTimeout;
    private clearAllClearTimeout;
    private dispatchEventOfType;
    private callListener;
    private debugLog;
}
export {};
